@@ -27,20 +27,20 @@ https://github.com/mattgwwalker/msg-extractor
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 __author__ = 'Destiny Peterson & Matthew Walker'
-__date__ = '2021-03-02'
-__version__ = '0.28.7'
+__date__ = '2022-01-16'
+__version__ = '0.29.1'
 
 import logging
 
-from extract_msg import constants
-from extract_msg.appointment import Appointment
-from extract_msg.attachment import Attachment
-from extract_msg.contact import Contact
-from extract_msg.exceptions import UnrecognizedMSGTypeError
-from extract_msg.message import Message
-from extract_msg.message_base import MessageBase
-from extract_msg.msg import MSGFile
-from extract_msg.prop import create_prop
-from extract_msg.properties import Properties
-from extract_msg.recipient import Recipient
-from extract_msg.utils import openMsg, properHex
+from . import constants
+from .appointment import Appointment
+from .attachment import Attachment
+from .contact import Contact
+from .exceptions import UnrecognizedMSGTypeError
+from .message import Message
+from .message_base import MessageBase
+from .msg import MSGFile
+from .prop import createProp
+from .properties import Properties
+from .recipient import Recipient
+from .utils import openMsg, properHex
