@@ -1,12 +1,10 @@
 import copy
 import logging
 import pprint
-import zlib
-
-from extract_msg import constants
-from extract_msg.utils import bytesToGuid, divide, properHex, roundUp
-
 
+from . import constants
+from .utils import bytesToGuid, divide, properHex, roundUp
+from compressed_rtf.crc32 import crc32
 
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
@@ -16,29 +14,25 @@ class Named(object):
     def __init__(self, msg):
         super(Named, self).__init__()
         self.__msg = msg
-        guid_stream = self._getStream('__substg1.0_00020102')
-        entry_stream = self._getStream('__substg1.0_00030102')
-        names_stream = self._getStream('__substg1.0_00040102')
-        guid_stream = self._getStream('__substg1.0_00020102', False) if guid_stream is None else guid_stream
-        entry_stream = self._getStream('__substg1.0_00030102', False) if entry_stream is None else entry_stream
-        names_stream = self._getStream('__substg1.0_00040102', False) if names_stream is None else names_stream
-        self.guid_stream = guid_stream
-        self.entry_stream = entry_stream
-        self.names_stream = names_stream
-        guid_stream_length = len(guid_stream)
-        entry_stream_length = len(entry_stream)
-        names_stream_length = len(names_stream)
-        # TODO guid stream parsing
-        guids = tuple([None, constants.GUID_PS_MAPI, constants.GUID_PS_PUBLIC_STRINGS] + [bytesToGuid(x) for x in divide(guid_stream, 16)])
-        # TODO entry_stream parsing
+        guidStream = self._getStream('__substg1.0_00020102') or self._getStream('__substg1.0_00020102', False)
+        entryStream = self._getStream('__substg1.0_00030102') or self._getStream('__substg1.0_00030102', False)
+        namesStream = self._getStream('__substg1.0_00040102') or self._getStream('__substg1.0_00040102', False)
+        self.guidStream = guidStream
+        self.entryStream = entryStream
+        self.namesStream = namesStream
+        guidStreamLength = len(guidStream)
+        entryStreamLength = len(entryStream)
+        namesStreamLength = len(namesStream)
+        guids = tuple([None, constants.GUID_PS_MAPI, constants.GUID_PS_PUBLIC_STRINGS] + [bytesToGuid(x) for x in divide(guidStream, 16)])
         entries = []
-        for x in divide(entry_stream, 8):
-            tmp = constants.STNP_ENT.unpack(x)
+        for rawStream in divide(entryStream, 8):
+            tmp = constants.STNP_ENT.unpack(rawStream)
             entry = {
                 'id': tmp[0],
                 'pid': tmp[2],
                 'guid_index': tmp[1] >> 1,
                 'pkind': tmp[1] & 1, # 0 if numerical, 1 if string
+                'rawStream': rawStream,
                 }
             entry['guid'] = guids[entry['guid_index']]
             entries.append(entry)
@@ -46,11 +40,11 @@ class Named(object):
         # Parse the names stream.
         names = {}
         pos = 0
-        while pos < names_stream_length:
-            name_length = constants.STNP_NAM.unpack(names_stream[pos:pos+4])[0]
-            pos += 4 # Move to the start of the
-            names[pos - 4] = names_stream[pos:pos+name_length].decode('utf_16_le') # Names are stored in the dictionary as the position they start at
-            pos += roundUp(name_length, 4)
+        while pos < namesStreamLength:
+            nameLength = constants.STNP_NAM.unpack(namesStream[pos:pos+4])[0]
+            pos += 4 # Move to the start of the entry.
+            names[pos - 4] = namesStream[pos:pos+nameLength].decode('utf-16-le') # Names are stored in the dictionary as the position they start at.
+            pos += roundUp(nameLength, 4)
 
         self.entries = entries
         self.__names = names
@@ -59,7 +53,7 @@ class Named(object):
         for entry in entries:
             streamID = properHex(0x8000 + entry['pid'])
             msg._registerNamedProperty(entry, entry['pkind'], names[entry['id']] if entry['pkind'] == constants.STRING_NAMED else None)
-            if msg.ExistsTypedProperty(streamID):
+            if msg.existsTypedProperty(streamID):
                 self.__properties.append(StringNamedProperty(entry, names[entry['id']], msg._getTypedData(streamID)) if entry['pkind'] == constants.STRING_NAMED else NumericalNamedProperty(entry, msg._getTypedData(streamID)))
         self.__propertiesDict = {}
         for property in self.__properties:
@@ -98,11 +92,11 @@ class Named(object):
         prop = self.getNamed(propertyName)
         return prop.data if prop is not None else None
 
-    def Exists(self, filename):
+    def exists(self, filename):
         """
         Checks if stream exists inside the named properties folder.
         """
-        return self.__msg.Exists([self.__dir, filename])
+        return self.__msg.exists([self.__dir, filename])
 
     def sExists(self, filename):
         """
@@ -153,7 +147,7 @@ class NamedAttachmentProperties(object):
         Informs the class of a named property that needs to be loaded.
         """
         streamID = properHex(0x8000 + entry['pid']).upper()
-        if self.__attachment.ExistsTypedProperty(streamID)[0]:
+        if self.__attachment.existsTypedProperty(streamID)[0]:
             data = self.__attachment._getTypedData(streamID)
             property = StringNamedProperty(entry, name, data) if _type == constants.STRING_NAMED else NumericalNamedProperty(entry, data)
             self.__properties.append(property)
@@ -183,8 +177,33 @@ class StringNamedProperty(object):
         self.__guidIndex = entry['guid_index']
         self.__guid = entry['guid']
         self.__namedPropertyID = entry['pid']
-        # WARNING From the way the documentation is worded, this SHOULD work, but it doesn't.
-        self.__streamID = 0x1000 + (zlib.crc32(name.lower().encode('utf-16-le')) ^ (self.__guidIndex << 1 | 1)) % 0x1F
+
+        # Finally got this to be correct after asking about it on a Microsoft
+        # forum. Apparently it uses the same CRC-32 as the Compressed RTF
+        # standard does, so we can just use the function defined in the
+        # compressed-rtf Python module.
+        #
+        # First thing to note is that the name should only ever be lowered if it
+        # is part of the PS_INTERNET_HEADERS property set **AND** it is
+        # generated by certain versions of Outlook. As such, a little bit of
+        # additional code will need to run to determine exactly what the stream
+        # ID should be if it is in that property set.
+        if self.__guid == constants.GUID_PS_INTERNET_HEADERS:
+            # To be sure if it needs to be lower the most effective method would
+            # be to just get the Stream ID and then check if the entry is in
+            # there. If it isn't, then check the regular case and see. If it is
+            # not in either... well, we don't use it for anything so it will
+            # just be a warning, and the Stream ID will be set to 0.
+            #
+            # TODO: Unfortunately, doing this will need to be put off until a
+            # different version, preferably after Python 2 support is removed,
+            # as this will require restructuring a lot of internal code. For now
+            # we just assume that it is lowercase.
+            self.__streamID = 0x1000 + (crc32(name.lower().encode('utf-16-le')) ^ (self.__guidIndex << 1 | 1)) % 0x1F
+
+        else:
+            # No special logic here to determine what to do.
+            self.__streamID = 0x1000 + (crc32(name.encode('utf-16-le')) ^ (self.__guidIndex << 1 | 1)) % 0x1F
         self.__data = data
 
     @property
@@ -215,6 +234,13 @@ class StringNamedProperty(object):
         """
         return self.__namedPropertyID
 
+    @property
+    def rawEntryStream(self):
+        """
+        The raw data used for the entry.
+        """
+        return self.__entry['rawStream']
+
     @property
     def streamID(self):
         """
@@ -237,8 +263,10 @@ class NumericalNamedProperty(object):
         self.__propertyID = properHex(entry['id'], 4).upper()
         self.__guidIndex = entry['guid_index']
         self.__namedPropertyID = entry['pid']
+        self.__guid = entry['guid']
         self.__streamID = 0x1000 + (entry['id'] ^ (self.__guidIndex << 1)) % 0x1F
         self.__data = data
+        self.__entry = entry
 
     @property
     def data(self):
